==> boxw-0.0.21.tar.gz: lib/src/box.dart <==
part of '../boxw.dart';

class BoxWInput extends StatelessWidget {
  const BoxWInput({
    super.key,
    this.title = '',
    this.onChanged,
    this.initial,
    this.controller,
    this.validator,
    this.leadingButton,
    this.trailingButton,
    this.prefixIcon,
    this.suffixIcon,
    this.obscureText = false,
    this.keyboardType,
    this.margin = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.padding = const EdgeInsets.symmetric(vertical: 12, horizontal: 12),
    this.style,
    this.titleStyle,
    this.titleAlign,
    this.floatingLabelAlignment,
    this.minLines = 1,
    this.maxLines,
    this.radius,
    this.borderColor,
    this.expands = false,
    this.enabled = true,
    this.autocorrect = false,
    this.errorText,
    this.maxLength,
    this.textAlign = TextAlign.center,
  });

  final String title;
  final String? initial;
  final TextEditingController? controller;
  final Function(String)? onChanged;
  final String? Function(String?)? validator;

  /// Use `IconButton` in most cases.
  final Widget? leadingButton;
  final Widget? trailingButton;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final bool obscureText;
  final TextInputType? keyboardType;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final TextStyle? style;
  final TextAlign? titleAlign;
  final TextStyle? titleStyle;
  final FloatingLabelAlignment? floatingLabelAlignment;
  final int? minLines;
  final int? maxLines;
  final double? radius;
  final Color? borderColor;
  final bool expands;
  final bool enabled;
  final bool autocorrect;
  final String? errorText;
  final int? maxLength;
  final TextAlign textAlign;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          leadingButton ?? SizedBox.fromSize(),
          Expanded(
            child: TextFormField(
              enabled: enabled,
              style: style,
              decoration: InputDecoration(
                contentPadding: padding,
                border: OutlineInputBorder(
                  borderSide: BorderSide(
                    color: borderColor ??
                        BoxWConfig.borderColor ??
                        Theme.of(context).colorScheme.outline,
                  ),
                  borderRadius: BorderRadius.circular(
                    radius ?? BoxWConfig.radius,
                  ),
                ),
                label: Text(
                  title,
                  overflow: TextOverflow.ellipsis,
                  textAlign: titleAlign,
                  style: titleStyle,
                ),
                floatingLabelAlignment: floatingLabelAlignment,
                errorText: errorText,
                isDense: true,
                prefixIcon: prefixIcon,
                suffixIcon: suffixIcon,
              ),
              controller: controller,
              initialValue: controller == null ? initial : null,
              onChanged: onChanged,
              validator: validator,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              obscureText: obscureText,
              keyboardType: keyboardType,
              minLines: minLines,
              maxLines: obscureText ? 1 : maxLines,
              expands: expands,
              autocorrect: autocorrect,
              maxLength: maxLength,
              textAlign: textAlign,
            ),
          ),
          trailingButton ?? SizedBox.fromSize(),
        ],
      ),
    );
  }
}

class BoxWDropdown<T> extends StatelessWidget {
  const BoxWDropdown({
    super.key,
    required this.items,
    required this.value,
    this.onChanged,
    this.selectedItemBuilder,
    this.title,
    this.borderColor,
    this.radius,
    this.titleStyle,
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
  });

  final String? title;
  final T value;
  final List<DropdownMenuItem<T>> items;
  final void Function(T? value)? onChanged;
  final List<Widget> Function(BuildContext)? selectedItemBuilder;
  final TextStyle? titleStyle;
  final double? radius;
  final Color? borderColor;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: padding,
      child: DropdownButtonFormField<T>(
        decoration: InputDecoration(
          isDense: true,
          border: OutlineInputBorder(
            borderSide: BorderSide(
              color: borderColor ??
                  BoxWConfig.borderColor ??
                  Theme.of(context).colorScheme.outline,
            ),
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
          ),
          label: title == null ? null : Text(title!, style: titleStyle),
        ),
        borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
        padding: EdgeInsets.zero,
        value: value,
        items: items,
        selectedItemBuilder: selectedItemBuilder,
        onChanged: onChanged,
      ),
    );
  }
}

class BoxWButton extends StatelessWidget {
  const BoxWButton({
    super.key,
    required this.child,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 10, horizontal: 4),
  });

  final Widget child;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: ElevatedButton(
        onPressed: onPressed,
        style: ElevatedButton.styleFrom(
          backgroundColor: backgroundColor,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
            side: BorderSide(
              color: borderColor ??
                  BoxWConfig.borderColor ??
                  Theme.of(context).colorScheme.outline,
            ),
          ),
        ),
        child: Padding(
          padding: padding,
          child: child,
        ),
      ),
    );
  }
}

class BoxWOutlinedButton extends StatelessWidget {
  const BoxWOutlinedButton({
    super.key,
    required this.child,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 10, horizontal: 4),
  });

  final Widget child;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: OutlinedButton(
        onPressed: onPressed,
        style: OutlinedButton.styleFrom(
          backgroundColor: backgroundColor,
          side: borderColor == null && BoxWConfig.borderColor == null
              ? null
              : BorderSide(
                  color: borderColor ??
                      BoxWConfig.borderColor ??
                      Theme.of(context).colorScheme.outline,
                ),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
          ),
        ),
        child: Padding(
          padding: padding,
          child: child,
        ),
      ),
    );
  }
}

class BoxWTextButton extends StatelessWidget {
  const BoxWTextButton({
    super.key,
    required this.child,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 0, horizontal: 12),
  });

  final Widget child;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: TextButton(
        onPressed: onPressed,
        style: TextButton.styleFrom(
          backgroundColor: backgroundColor,
        ),
        child: Padding(
          padding: padding,
          child: child,
        ),
      ),
    );
  }
}

class BoxWRect extends StatelessWidget {
  const BoxWRect({
    super.key,
    required this.child,
    this.borderColor,
    this.backgroundColor,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.alignment = Alignment.centerLeft,
    this.radius,
    this.width,
    this.height,
  });

  final Widget child;
  final Color? borderColor;
  final Color? backgroundColor;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final AlignmentGeometry alignment;
  final double? radius;
  final double? width;
  final double? height;

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: alignment,
      width: width ?? double.infinity,
      height: height,
      margin: margin,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
        border: borderColor != null || BoxWConfig.borderColor != null
            ? Border.all(
                color: borderColor ??
                    BoxWConfig.borderColor ??
                    Theme.of(context).colorScheme.outline,
              )
            : null,
        color: backgroundColor,
      ),
      child: Padding(
        padding: padding,
        child: child,
      ),
    );
  }
}

class BoxWCard extends StatelessWidget {
  const BoxWCard({
    super.key,
    required this.child,
    this.margin,
    this.borderColor,
    this.backgroundColor,
    this.radius,
  });

  final Widget child;
  final EdgeInsetsGeometry? margin;
  final Color? borderColor;
  final Color? backgroundColor;
  final double? radius;

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 0,
      color: backgroundColor ?? Theme.of(context).colorScheme.surfaceVariant,
      margin: margin,
      shape: RoundedRectangleBorder(
        side: BorderSide(
          color: borderColor ??
              BoxWConfig.borderColor ??
              Theme.of(context).colorScheme.outline,
        ),
        borderRadius:
            BorderRadius.all(Radius.circular(radius ?? BoxWConfig.radius)),
      ),
      child: child,
    );
  }
}

class BoxWIconTextButton extends StatelessWidget {
  const BoxWIconTextButton({
    super.key,
    required this.icon,
    required this.title,
    this.trailing,
    this.onPressed,
    this.titlePadding = const EdgeInsets.symmetric(horizontal: 12, vertical: 0),
    this.padding = const EdgeInsets.symmetric(horizontal: 4, vertical: 4),
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.isNavigateTrailing = false,
  });

  final Widget icon;
  final Widget title;
  final bool isNavigateTrailing;
  final Widget? trailing;
  final void Function()? onPressed;
  final EdgeInsetsGeometry titlePadding;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;

  @override
  Widget build(BuildContext context) {
    return BoxWTextButton(
      padding: padding,
      margin: margin,
      onPressed: onPressed,
      child: Row(
        children: [
          icon,
          Expanded(
            child: Padding(
              padding: titlePadding,
              child: title,
            ),
          ),
          trailing ??
              SizedBox(
                width: 30,
                child: isNavigateTrailing
                    ? const Icon(Icons.navigate_next_outlined)
                    : null,
              ),
        ],
      ),
    );
  }
}

class BoxWTextIcon extends StatelessWidget {
  const BoxWTextIcon({
    super.key,
    required this.icon,
    required this.title,
    this.trailing,
    this.onPressed,
    this.isNavigateTrailing = false,
    this.borderColor,
    this.backgroundColor,
    this.subTitle,
    this.titlePadding = const EdgeInsets.symmetric(horizontal: 16, vertical: 0),
    this.padding = const EdgeInsets.symmetric(horizontal: 16, vertical: 8),
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
  });

  final Widget icon;
  final Widget title;
  final EdgeInsetsGeometry titlePadding;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final String? subTitle;
  final Color? borderColor;
  final Color? backgroundColor;
  final bool isNavigateTrailing;
  final Widget? trailing;
  final void Function()? onPressed;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: InkWell(
        onTap: onPressed,
        borderRadius: BorderRadius.circular(12),
        child: BoxWRect(
          margin: EdgeInsets.zero,
          borderColor: borderColor,
          backgroundColor: backgroundColor,
          padding: padding,
          child: Row(
            children: [
              icon,
              Expanded(
                child: Padding(
                  padding: titlePadding,
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      title,
                      if (subTitle != null) ...[
                        const SizedBox(height: 3),
                        Text(
                          subTitle!,
                          textAlign: TextAlign.left,
                          style:
                              const TextStyle(fontSize: 12, color: Colors.grey),
                        ),
                      ]
                    ],
                  ),
                ),
              ),
              trailing ??
                  (isNavigateTrailing
                      ? const SizedBox(
                          width: 30, child: Icon(Icons.navigate_next_outlined))
                      : const SizedBox.shrink()),
            ],
          ),
        ),
      ),
    );
  }
}

==> boxw-0.0.24.tar.gz: lib/src/box.dart <==
part of '../boxw.dart';

class BoxWInput extends StatelessWidget {
  const BoxWInput({
    super.key,
    this.title = '',
    this.onChanged,
    this.initial,
    this.controller,
    this.validator,
    this.leadingButton,
    this.trailingButton,
    this.prefixIcon,
    this.suffixIcon,
    this.obscureText = false,
    this.keyboardType,
    this.margin = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.padding = const EdgeInsets.symmetric(vertical: 12, horizontal: 12),
    this.style,
    this.titleStyle,
    this.titleAlign,
    this.floatingLabelAlignment,
    this.minLines = 1,
    this.maxLines,
    this.radius,
    this.borderColor,
    this.expands = false,
    this.enabled = true,
    this.autocorrect = false,
    this.errorText,
    this.maxLength,
    this.textAlign = TextAlign.center,
  });

  final String title;
  final String? initial;
  final TextEditingController? controller;
  final Function(String)? onChanged;
  final String? Function(String?)? validator;

  /// Use `IconButton` in most cases.
  final Widget? leadingButton;
  final Widget? trailingButton;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final bool obscureText;
  final TextInputType? keyboardType;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final TextStyle? style;
  final TextAlign? titleAlign;
  final TextStyle? titleStyle;
  final FloatingLabelAlignment? floatingLabelAlignment;
  final int? minLines;
  final int? maxLines;
  final double? radius;
  final Color? borderColor;
  final bool expands;
  final bool enabled;
  final bool autocorrect;
  final String? errorText;
  final int? maxLength;
  final TextAlign textAlign;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          leadingButton ?? SizedBox.fromSize(),
          Expanded(
            child: TextFormField(
              enabled: enabled,
              style: style,
              decoration: InputDecoration(
                contentPadding: padding,
                border: OutlineInputBorder(
                  borderSide: BorderSide(
                    color: borderColor ??
                        BoxWConfig.borderColor ??
                        Theme.of(context).colorScheme.outline,
                  ),
                  borderRadius: BorderRadius.circular(
                    radius ?? BoxWConfig.radius,
                  ),
                ),
                label: Text(
                  title,
                  overflow: TextOverflow.ellipsis,
                  textAlign: titleAlign,
                  style: titleStyle,
                ),
                floatingLabelAlignment: floatingLabelAlignment,
                errorText: errorText,
                isDense: true,
                prefixIcon: prefixIcon,
                suffixIcon: suffixIcon,
              ),
              controller: controller,
              initialValue: controller == null ? initial : null,
              onChanged: onChanged,
              validator: validator,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              obscureText: obscureText,
              keyboardType: keyboardType,
              minLines: minLines,
              maxLines: obscureText ? 1 : maxLines,
              expands: expands,
              autocorrect: autocorrect,
              maxLength: maxLength,
              textAlign: textAlign,
            ),
          ),
          trailingButton ?? SizedBox.fromSize(),
        ],
      ),
    );
  }
}

class BoxWDropdown<T> extends StatelessWidget {
  const BoxWDropdown({
    super.key,
    required this.items,
    required this.value,
    this.onChanged,
    this.selectedItemBuilder,
    this.title,
    this.borderColor,
    this.radius,
    this.titleStyle,
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
  });

  final String? title;
  final T value;
  final List<DropdownMenuItem<T>> items;
  final void Function(T? value)? onChanged;
  final List<Widget> Function(BuildContext)? selectedItemBuilder;
  final TextStyle? titleStyle;
  final double? radius;
  final Color? borderColor;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: padding,
      child: DropdownButtonFormField<T>(
        decoration: InputDecoration(
          isDense: true,
          border: OutlineInputBorder(
            borderSide: BorderSide(
              color: borderColor ??
                  BoxWConfig.borderColor ??
                  Theme.of(context).colorScheme.outline,
            ),
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
          ),
          label: title == null ? null : Text(title!, style: titleStyle),
        ),
        borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
        padding: EdgeInsets.zero,
        value: value,
        items: items,
        selectedItemBuilder: selectedItemBuilder,
        onChanged: onChanged,
      ),
    );
  }
}

class BoxWButton extends StatelessWidget {
  const BoxWButton({
    super.key,
    required this.child,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 10, horizontal: 4),
  });

  final Widget child;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: ElevatedButton(
        onPressed: onPressed,
        style: ElevatedButton.styleFrom(
          backgroundColor: backgroundColor,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
            side: BorderSide(
              color: borderColor ??
                  BoxWConfig.borderColor ??
                  Theme.of(context).colorScheme.outline,
            ),
          ),
        ),
        child: Padding(
          padding: padding,
          child: child,
        ),
      ),
    );
  }
}

class BoxWOutlinedButton extends StatelessWidget {
  const BoxWOutlinedButton({
    super.key,
    required this.child,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 10, horizontal: 4),
  });

  final Widget child;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: OutlinedButton(
        onPressed: onPressed,
        style: OutlinedButton.styleFrom(
          backgroundColor: backgroundColor,
          side: borderColor == null && BoxWConfig.borderColor == null
              ? null
              : BorderSide(
                  color: borderColor ??
                      BoxWConfig.borderColor ??
                      Theme.of(context).colorScheme.outline,
                ),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
          ),
        ),
        child: Padding(
          padding: padding,
          child: child,
        ),
      ),
    );
  }
}

class BoxWTextButton extends StatelessWidget {
  const BoxWTextButton({
    super.key,
    required this.child,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 0, horizontal: 12),
  });

  final Widget child;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: TextButton(
        onPressed: onPressed,
        style: TextButton.styleFrom(
          backgroundColor: backgroundColor,
        ),
        child: Padding(
          padding: padding,
          child: child,
        ),
      ),
    );
  }
}

class BoxWCard extends StatelessWidget {
  const BoxWCard({
    super.key,
    required this.child,
    this.margin,
    this.borderColor,
    this.backgroundColor,
    this.radius,
  });

  final Widget child;
  final EdgeInsetsGeometry? margin;
  final Color? borderColor;
  final Color? backgroundColor;
  final double? radius;

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 0,
      color: backgroundColor ?? Theme.of(context).colorScheme.surfaceVariant,
      margin: margin,
      shape: RoundedRectangleBorder(
        side: BorderSide(
          color: borderColor ??
              BoxWConfig.borderColor ??
              Theme.of(context).colorScheme.outline,
        ),
        borderRadius:
            BorderRadius.all(Radius.circular(radius ?? BoxWConfig.radius)),
      ),
      child: child,
    );
  }
}

class BoxWIconTextButton extends StatelessWidget {
  const BoxWIconTextButton({
    super.key,
    required this.icon,
    required this.title,
    this.trailing,
    this.onPressed,
    this.titlePadding = const EdgeInsets.symmetric(horizontal: 12, vertical: 0),
    this.padding = const EdgeInsets.symmetric(horizontal: 4, vertical: 4),
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.isNavigateTrailing = false,
  });

  final Widget icon;
  final Widget title;
  final bool isNavigateTrailing;
  final Widget? trailing;
  final void Function()? onPressed;
  final EdgeInsetsGeometry titlePadding;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;

  @override
  Widget build(BuildContext context) {
    return BoxWTextButton(
      padding: padding,
      margin: margin,
      onPressed: onPressed,
      child: Row(
        children: [
          icon,
          Expanded(
            child: Padding(
              padding: titlePadding,
              child: title,
            ),
          ),
          trailing ??
              SizedBox(
                width: 30,
                child: isNavigateTrailing
                    ? const Icon(Icons.navigate_next_outlined)
                    : null,
              ),
        ],
      ),
    );
  }
}

class BoxWTextIcon extends StatelessWidget {
  const BoxWTextIcon({
    super.key,
    required this.icon,
    required this.title,
    this.trailing,
    this.onPressed,
    this.isNavigateTrailing = false,
    this.borderColor,
    this.backgroundColor,
    this.subTitle,
    this.titlePadding = const EdgeInsets.symmetric(horizontal: 16, vertical: 0),
    this.padding = const EdgeInsets.symmetric(horizontal: 16, vertical: 8),
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
  });

  final Widget icon;
  final Widget title;
  final EdgeInsetsGeometry titlePadding;
  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final String? subTitle;
  final Color? borderColor;
  final Color? backgroundColor;
  final bool isNavigateTrailing;
  final Widget? trailing;
  final void Function()? onPressed;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: InkWell(
        onTap: onPressed,
        borderRadius: BorderRadius.circular(12),
        child: BoxWRect(
          margin: EdgeInsets.zero,
          borderColor: borderColor,
          backgroundColor: backgroundColor,
          padding: padding,
          child: Row(
            children: [
              icon,
              Expanded(
                child: Padding(
                  padding: titlePadding,
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      title,
                      if (subTitle != null) ...[
                        const SizedBox(height: 3),
                        Text(
                          subTitle!,
                          textAlign: TextAlign.left,
                          style:
                              const TextStyle(fontSize: 12, color: Colors.grey),
                        ),
                      ]
                    ],
                  ),
                ),
              ),
              trailing ??
                  (isNavigateTrailing
                      ? const SizedBox(
                          width: 30, child: Icon(Icons.navigate_next_outlined))
                      : const SizedBox.shrink()),
            ],
          ),
        ),
      ),
    );
  }
}

class BoxWRect extends StatelessWidget implements BoxWShape {
  const BoxWRect({
    super.key,
    required this.child,
    this.borderWidth = 1,
    this.borderColor,
    this.backgroundColor,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.alignment = Alignment.centerLeft,
    this.radius,
    this.width,
    this.height,
  });

  final Widget child;
  final Color? borderColor;
  final Color? backgroundColor;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final AlignmentGeometry alignment;
  final double? radius;
  final double? width;
  final double? height;
  final double borderWidth;

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: alignment,
      width: width ?? double.infinity,
      height: height,
      margin: margin,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
        border: borderColor != null || BoxWConfig.borderColor != null
            ? Border.all(
                width: borderWidth,
                color: borderColor ??
                    BoxWConfig.borderColor ??
                    Theme.of(context).colorScheme.outline,
              )
            : null,
        color: backgroundColor,
      ),
      child: Padding(
        padding: padding,
        child: child,
      ),
    );
  }
}

class BoxWCircle extends StatelessWidget implements BoxWShape {
  const BoxWCircle({
    super.key,
    required this.child,
    this.borderColor,
    this.borderWidth = 1.0,
    this.backgroundColor,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.alignment = Alignment.centerLeft,
    this.width,
    this.height,
  });

  final Widget child;
  final Color? borderColor;
  final Color? backgroundColor;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final AlignmentGeometry alignment;
  final double? width;
  final double? height;
  final double borderWidth;

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: alignment,
      width: width ?? double.infinity,
      height: height,
      margin: margin,
      padding: padding,
      decoration: BoxDecoration(
        shape: BoxShape.circle,
        border: borderColor != null || BoxWConfig.borderColor != null
            ? Border.all(
                width: borderWidth,
                color: borderColor ??
                    BoxWConfig.borderColor ??
                    Theme.of(context).colorScheme.outline,
              )
            : null,
        color: backgroundColor,
      ),
      child: child,
    );
  }
}
